# -*- coding:utf-8 -*-
#
# Copyright (C) 2019-2020 Alibaba Group Holding Limited


from __future__ import print_function
from aostools import *


class Cpu(Command):
    common = True
    helpSummary = "list support cpu models"
    helpUsage = """
%prog [option]
"""
    helpDescription = """
list support cpu models
"""

    def _Options(self, p):
        p.add_option('-c', '--csky',
                     dest='csky', action='store_true',
                     help='list csky cpu models')
        p.add_option('-a', '--arm',
                     dest='arm', action='store_true',
                     help='list arm cpu models')
        p.add_option('-r', '--riscv',
                     dest='riscv', action='store_true',
                     help='list riscv cpu models')

    def Execute(self, opt, args):
        ck_cpu = globals()['CkcoreCPU']
        arm_cpu = globals()['ArmCPU']
        rv_cpu = globals()['RiscvCPU']
        if not (opt.csky or opt.arm or opt.riscv):
            print('### csky cpu:')
            print(ck_cpu)
            print('### riscv cpu:')
            print(rv_cpu)
            print('### arm cpu:')
            print(arm_cpu)
            return
        if opt.csky:
            print('### csky cpu:')
            print(ck_cpu)
        if opt.arm:
            print('### arm cpu:')
            print(arm_cpu)
        if opt.riscv:
            print('### riscv cpu:')
            print(rv_cpu)



# -*- coding:utf-8 -*-
#
# Copyright (C) 2019-2020 Alibaba Group Holding Limited


from __future__ import print_function

from aostools import *
import os
import sys
import struct
import time
import zlib

try:
    import serial
except ImportError:
    print("Pyserial is not installed for %s. Check the README for installation instructions." % (sys.executable))
    raise

try:
    import serial.tools.list_ports as list_ports
except ImportError:
    print("The installed version (%s) of pyserial appears to be too old for esptool.py (Python interpreter %s). "
          "Check the README for installation instructions." % (sys.VERSION, sys.executable))
    raise
except Exception:
    if sys.platform == "darwin":
        list_ports = None
    else:
        raise

def print_overwrite(message, last_line=False):
    if sys.stdout.isatty():
        print("\r%s" % message, end='\n' if last_line else '')
        sys.stdout.flush()
    else:
        print(message)

class UnsupportedCommandError(RuntimeError):
    """
    Wrapper class for when ROM loader returns an invalid command response.
    Usually this indicates the loader is running in Secure Download Mode.
    """
    def __init__(self, cct, op):
        msg = "Invalid (unsupported) command 0x%x" % op
        RuntimeError.__init__(self, msg)

def to_size(size):
    if size < 1024:
        return '{}'.format(size)
    elif size < 1024 * 1024:
        return '%6.1fKB' % (size / 1024)
    elif size < 1024 * 1024 * 1024:
        return '%6.1fMB'% (size / 1024 / 1024)
    elif size < 1024 * 1024 * 1024 * 1024:
        return '%6.1fGB' % (size / 1024 / 1024 / 1024.0)
    return '{}'.format(size)

DEFAULT_TIMEOUT = 3      # timeout for most flash operations
MAX_TIMEOUT = 300        # longest any command can run

class CCT:
    DEFAULT_PORT = "/dev/ttyUSB0"
    CHIP_ROM_BAUD = 115200

    def __init__(self, port=DEFAULT_PORT, baud=CHIP_ROM_BAUD, trace_enabled=True):
        self._port = None
        self._trace_enabled = trace_enabled
        self._last_trace = time.time()
        if port == None and list_ports:
            port_list = list(list_ports.comports())
            for i in port_list:
                if i.hwid != 'n/a':
                    port = i.device
                    break
        if not port:
            print("Not found serial port.")
            exit()
        try:
            self._port = serial.Serial(port, baud)
        except:
            raise Exception("could not open serial port {}".format(port))

    def read(self, count):
        return self._port.read(count)

    def write(self, data):
        self._port.write(data)

    def command(self, cmd, param, data=b'', timeout=DEFAULT_TIMEOUT):
        saved_timeout = self._port.timeout
        new_timeout = min(timeout, MAX_TIMEOUT)
        if new_timeout != saved_timeout:
            self._port.timeout = new_timeout

        try:
            c = struct.pack('!BBH', cmd, param, len(data)) + data
            self.write(c)
            v = self.read(4)
            if len(v) == 4:
                data = struct.unpack('!BBH', v)
                payload = self.read(data[2])

                if data[0] == cmd and data[1] >= 0:
                    return True, data, payload

                if data[1] == 0xFD or data[1] == 0xFF:
                    raise UnsupportedCommandError(self, cmd)

            if len(v) >= 4 and data[0] != cmd:
                raise UnsupportedCommandError(self, cmd)
        finally:
            if new_timeout != saved_timeout:
                self._port.timeout = saved_timeout

        return False, [], None


    def connect(self, timeout=30):
        start_ticks = time.time()
        print_count = 0
        while True:
            now_tick = time.time()
            if now_tick - start_ticks > timeout:
                break

            ret, v, _ = self.command(0x10, 0xEF, struct.pack('!BBBB', 0x43, 0x53, 0x4B, 0x59), timeout=0.1)
            if ret and v[1] == 1:
                if print_count != 0:
                    print()
                self.trace('connect succuss!')

                return True
            else:
                if print_count == 0:
                    print("Wait ", end='')
                else:
                    print(".", end='')
                print_count += 1
                sys.stdout.flush()
        return False


    def get_version(self):
        ret, v, _ = self.command(0x01, 0x00)
        if ret:
            print('CCT Version: %d' % v[1])


    def get_timeout(self):
        ret, v, _ = self.command(0x0A, 0x00)
        if ret:
            print(v)


    def get_device_id(self, param):
        ret, v, _ = self.command(0x0B, param)
        if ret:
            if v[1] == 0xE0:
                self.trace('device id not exist.')

    def get_memory_info(self):
        mem = {}
        ret, v, data = self.command(0x11, 0x00)
        if ret and v[1] == 1:
            length = v[2]

            idx = 0
            for _ in range(0, length // 6):
                vv = struct.unpack('!BBI', data[idx : idx + 6])
                TY = vv[0]
                if TY == 0x00:
                    mem['recv'] = {'type': vv[0], 'id': vv[1], 'size': vv[2], 'desc': '协议接收缓冲区'}
                elif TY == 0x01:
                    mem['send'] = {'type': vv[0], 'id': vv[1], 'size': vv[2], 'desc': '协议发送缓冲区'}
                elif TY == 0x02:
                    mem['ram{}'.format(vv[1])] = {'type': vv[0], 'id': vv[1], 'size': vv[2], 'desc': 'SRAM'}
                elif TY == 0x03:
                    mem['eflash{}'.format(vv[1])] = {'type': vv[0], 'id': vv[1], 'size': vv[2], 'desc': 'EFlash'}
                elif TY == 0x10:
                    mem['ddr{}'.format(vv[1])] = {'type': vv[0], 'id': vv[1], 'size': vv[2], 'desc': 'SDRAM/DDR'}
                elif TY == 0x11:
                    mem['nflash{}'.format(vv[1])] = {'type': vv[0], 'id': vv[1], 'size': vv[2], 'desc': 'NOR Flash'}
                elif TY == 0x12:
                    mem['sflash{}'.format(vv[1])] = {'type': vv[0], 'id': vv[1], 'size': vv[2], 'desc': 'SPI Flash'}
                elif TY == 0x13:
                    mem['nand{}'.format(vv[1])] = {'type': vv[0], 'id': vv[1], 'size': vv[2], 'desc': 'Nand Flash'}
                elif TY == 0x14:
                    mem['sd{}'.format(vv[1])] = {'type': vv[0], 'id': vv[1], 'size' : vv[2] * 512, 'desc': 'SD Flash'}
                elif TY == 0x15:
                    mem['emmc{}'.format(vv[1])] = {'type': vv[0], 'id': vv[1], 'size': vv[2] * 512, 'desc': 'eMMC Flash'}
                idx += 6

        return mem


    def download_init(self, Type, ID, offset, length):
        # Cmd	Param	Len16_H	Len16_L	    Data, Len=(Len16_H<<8 & Len16_L)
        # 0x12	0x00	0x00	1+1+4+4+N	1:Type 1: ID, 4: Offset, 4: Size, N=Data Name
        # 0x12	0x01	0x00	4+4+N	     4: Address, 4: Size, N=Image Name
        c = struct.pack('!BBII', Type, ID, offset, length)
        ret, v, _ = self.command(0x12, 0x00, c)

        if ret and v[1] == 1 and v[2] == 0:
            self.trace('download_init type=%d, id=%d, offset=%d, lenght=%d', Type, ID, offset, length)
        return ret

    def download_data(self, flags, data):
        # Cmd	Param	Len16_H	Len16_L	Data, Len=(Len16_H<<8 & Len16_L)
        # 0x13	0x00~03	N	            N: (1~SoC Buffer Size)
        ret, v, _ = self.command(0x13, flags, data)
        if ret:
            if v[1] == flags:
                pass
            else:
                if v[1] == 0xE0:
                    self.trace("写入错误")
                elif v[1] == 0xE1:
                    self.trace("单数据包长度过大")
                elif v[1] == 0xE1:
                    self.trace("超过请求下载数据长度")
                return False

        return ret

    def download_file(self, offset, filename, Type=2, ID=0, block_size=512, compress=False):
        ret = False
        with open(filename, "rb") as F:
            file_size = 0
            if compress:
                while True:
                    data = F.read(block_size)
                    if not data:
                        break
                    data = zlib.compress(data, 9)
                    file_size += len(data)
                    full_size = F.tell()
                print("%d => %d %3.2f%%" % (full_size, file_size, file_size * 100.0 / full_size))

            else:
                F.seek(0, 2)
                file_size = F.tell()

            self.download_init(Type, ID, offset, file_size)

            offset = 0
            F.seek(0, 0)
            while True:
                data = F.read(block_size)
                data_len = len(data)
                if data_len > 0:
                    if compress:
                        uncdata = data
                        data = zlib.compress(uncdata, 9)
                        data_len = len(data)
                    if offset == 0:
                        ret = self.download_data(0x03, data)
                    elif offset + data_len < file_size:
                        ret = self.download_data(0x02, data)
                    else:
                        ret = self.download_data(0x00, data)
                    offset += data_len
                else:
                    break
                print_overwrite('Writing at 0x%08x... (%d%%)' % (offset, 100 * (offset / file_size)))
            print_overwrite("File {} download success.".format(filename), last_line=True)

        return ret

    def verify_data(self, verify_code):
        pass
        # c = struct.pack('!BBII', 2, 0, 0x00000)
        # ret, v, _ = self.command(0x18, 0x00, c)

    def read_memory(self, address, size=512):
        # 0x26	0x00	0x00	1+1+4+2	1:Type 1: ID, 4: Offset, 2: Size
        # 0x26	0x01	0x00	4+2	4: Address, 2: Size
        c = struct.pack('!II', address, size)
        ret, v, _ = self.command(0x12, 0x01, c)
        if ret:
            print(v)


    def run_image(self, offset):
        # Cmd	Param	Len16_H	Len16_L	   Data, Len=(Len16_H<<8 & Len16_L)
        # 0x18	0x00	0x00	1+1+4+N	   1:Type 1: ID, 4: Offset
        #                                  N: Run parameters, SoC will load to (P0-P3)
        c = struct.pack('!BBI', 2, 0, offset)
        ret, v, _ = self.command(0x18, 0x00, c)
        if ret:
            if v[1] == 0x01:
                self.trace('已准备好，立即运行!')
            elif v[1] == 0xE1:
                self.trace("设备不存在或地址不合法")
            elif v[1] == 0xE2:
                self.trace("源不可XIP执行")
            elif v[1] == 0xE3:
                self.trace("源读取失败")

        return ret and v[1] == 1


    def trace(self, message, *format_args):
        if self._trace_enabled:
            now = time.time()
            try:

                delta = now - self._last_trace
            except AttributeError:
                delta = 0.0
            self._last_trace = now
            prefix = "TRACE +%.3f " % delta
            print(prefix + (message % format_args))

class Cct(Command):
    common = True
    helpSummary = "Bootrom CCT tools"
    helpUsage = """
%prog <list|download> [param]
"""
    helpDescription = """
Bootrom CCT tools
cct read <address> <size>
cct write <type> <id> <offset> <filename>
"""

    def Execute(self, opt, args):
        if len(args) < 1:
            self.Usage()

        cct = CCT(opt.uart)
        cct._trace_enabled = opt.debug
        if not cct.connect():
            return
        cct.get_version()
        mem = cct.get_memory_info()

        if args[0] == 'list':
            print("memory device list:")
            for k, v in mem.items():
                if v['type'] > 1:
                    # print("  dev = %-5s, size = %s, type = %2d, id = %d, desc = %s" % (k, to_size(v['size']), v['type'], v['id'], v['desc']))
                    print("  dev = %-5s, size = %s" % (k, to_size(v['size'])))
        elif args[0] == 'download':
            if os.path.exists(opt.file):
                if opt.device in mem:
                    opt_type = mem[opt.device]['type']
                    opt_id = mem[opt.device]['id']
                    print("Send file '%s' to %d:%d ..." % (opt.file, opt_type, opt_id))
                    cct.download_file(opt.offset, opt.file, opt_type, opt_id, opt.block, opt.compress)
                    cct.run_image(opt.offset)
                else:
                    print('Device `%s` not exists, please select' % opt.device)
            else:
                print("file %s not exists" % opt.file)


    def _Options(self, p):
        p.add_option('-u',
                     dest='uart', action='store', type='str', default=None,
                     help='CCT serial port device')
        p.add_option('-f',
                     dest='file', action='store', type='str', default='',
                     help='')
        p.add_option('-o',
                     dest='offset', action='store', type='int', default=0,
                     help='Device start address')
        p.add_option('-b', '--block',
                     dest='block', action='store', type='int', default=512,
                     help='')
        p.add_option('-d', '--device',
                     dest='device', action='store', type='str', default=None,
                     help='Device name')
        p.add_option('-c', '--compress',
                     dest='compress', action='store_true', default=False,
                     help='')
        p.add_option('-D', '--debug',
                     dest='debug', action='store_true', default=False,
                     help='Enable debbug trace info')



# -*- coding:utf-8 -*-
#
# Copyright (C) 2019-2020 Alibaba Group Holding Limited


from __future__ import print_function

import os
import stat
import tarfile
import subprocess
import platform
import codecs

from .tools import *
from .gitproject import *


all_toolchain_url = {
    'Win32':{
    'csky-abiv2-elf': '',
    'riscv64-unknown-elf': '',
    'arm-none-eabi': 'https://gitee.com/alios-things/gcc-arm-none-eabi-win32.git -b aos'
    }, 
    'Linux32':{
    'csky-abiv2-elf': 'http://yoctools.oss-cn-beijing.aliyuncs.com/csky-elfabiv2-tools-i386-minilibc-20200407.tar.gz',
    'riscv64-unknown-elf': 'http://yoctools.oss-cn-beijing.aliyuncs.com/riscv64-elf-i386-20200820.tar.gz',
    'arm-none-eabi': ''
    }, 
    'Linux64':{
    'csky-abiv2-elf': 'http://yoctools.oss-cn-beijing.aliyuncs.com/csky-elfabiv2-tools-x86_64-minilibc-20200407.tar.gz',
    'riscv64-unknown-elf': 'http://yoctools.oss-cn-beijing.aliyuncs.com/riscv64-elf-x86_64-20200820.tar.gz',
    'arm-none-eabi': 'https://gitee.com/alios-things/gcc-arm-none-eabi-linux.git -b aos'
    }, 
    'OSX':{
    'csky-abiv2-elf': '',
    'riscv64-unknown-elf': '',
    'arm-none-eabi': 'https://gitee.com/alios-things/gcc-arm-none-eabi-osx.git -b aos'
    }
}


class ToolchainYoC:
    def __init__(self):
        self.basepath = home_path('.aliot')

    def download(self, arch):
        toolchain_path = os.path.join(self.basepath, arch)

        host_os = get_host_os()
        if arch not in all_toolchain_url[host_os]:
            put_string("Can not find toolchain for %s!" % arch)
            return

        if os.path.exists(toolchain_path):
            put_string("removing corrupt directory %s." % toolchain_path)
            try:
                shutil.rmtree(toolchain_path)
            except:
                delete_ret = 1
                if get_host_os() == 'Win32':
                    delete_ret = os.system("cmd /c rmdir /s /q %s" % toolchain_path) >> 8
                if delete_ret != 0:
                    put_string("Delete %s failed. Please delete it manually." % toolchain_path)
        
        toolchain_url = all_toolchain_url[host_os][arch]
        if not toolchain_url:
            put_string("Url is empty!")
            return
        put_string("Start to download toolchain: %s. \nPlease be patient for a few minutes..." % arch)
        pattern = re.compile(r'(.*)\s+\-b\s+(.*)')
        branch = "master"
        match = pattern.match(toolchain_url)
        if match:
            toolchain_url = match.group(1)
            branch = match.group(2)
        if toolchain_url.endswith('.git'):
            prj = GitRepo(toolchain_path, toolchain_url)
            put_string("clone %s -b %s ..." % (toolchain_url, branch))
            prj.pull(branch, None)
        else:
            if host_os == "Win32":
                tar_path = os.path.join(os.environ.get('TEMP'), os.path.basename(toolchain_url))
            else:
                tar_path = os.path.join('/tmp', os.path.basename(toolchain_url))
            wget(toolchain_url, tar_path)
            put_string("")
            put_string("Start install, wait half a minute please.")
            if tar_path.endswith('.bz2'):
                with tarfile.open(tar_path, 'r:bz2') as tar:
                    tar.extractall(toolchain_path)
            elif tar_path.endswith('.gz'):
                with tarfile.open(tar_path, 'r:gz') as tar:
                    tar.extractall(toolchain_path)
            else:
                put_string("%s extra not support." % tar_path)
                os.remove(tar_path)
                return

            os.remove(tar_path)
        put_string("Congratulations!")

    def check_toolchain(self, arch='csky-abiv2-elf', verbose=0):
        bin_file = self.check_program(arch)
        if not bin_file:
            self.download(arch)
            bin_file = self.check_program(arch)
        else:
            if verbose == 1:
                put_string('warn: the toolchains was installed already, path = %s.' % bin_file)
        return bin_file

    def check_program(self, arch='csky-abiv2-elf'):
        """ check PATH, ~/.aliot/<arch>/bin,  ~/.aliot/<arch>/main/bin"""
        path = which(arch + '-gcc')
        if arch.startswith('arm-none-eabi'):
            # arm-none-eabi toolchain updated!
            path = ""

        if not path:
            path = os.path.join(self.basepath, arch, "bin", arch + '-gcc')
            path = which(path)
            if not path:
                path = os.path.join(self.basepath, arch, "main", "bin", arch + '-gcc')
                path = which(path)
        return path

    def uninstall_toolchain(self, arch='csky-abiv2-elf'):
        path = which(arch + '-gcc')
        if arch.startswith('arm-none-eabi'):
            # arm-none-eabi toolchain updated!
            path = ""
        if not path:
            path = os.path.join(self.basepath, arch, "bin", arch + '-gcc')
            path = which(path)
            if not path:
                path = os.path.join(self.basepath, arch, "main", "bin", arch + '-gcc')
                path = which(path)
            if path:
                shutil.rmtree(os.path.join(self.basepath, arch))
                put_string("The toolchain \"%s\" is uninstalled." % path)
            else:
                put_string("The toolchain \"%s\" is not existed." % arch)
        else:
            put_string("The toolchain \"%s\" is in your PATH!\nPlease remove it manually if you really want." % path)



# -*- coding:utf-8 -*-
#
# Copyright (C) 2019-2020 Alibaba Group Holding Limited


from __future__ import print_function
import os
from aostools import *


class Sdk(Command):
    common = True
    helpSummary = "Generate current solution SDK"
    helpUsage = """
%prog
"""
    helpDescription = """
generate current solution SDK.
"""
    # def _Options(self, p):
    #     p.add_option('-a', '--all',
    #                  dest='show_all', action='store_true',
    #                  help='show the complete list of commands')

    def Execute(self, opt, args):
        yoc = YoC()
        solution = yoc.getSolution()
        if solution:
            solution.install()
            pack = solution.solution_component.pack
            if pack:
                for libname in solution.libs:
                    if libname not in pack.build_config.libs:
                        pack.build_config.libs.append(libname)

                pack.build_config.libpath.append('aos_sdk/lib')
                pack.build_config.include.append('aos_sdk/include')
                pack.depends = {}
                pack.hw_info.reset()
                pack.hw_info.cpu_name = solution.cpu_name
                # pack.hw_info.ld_script = 'aos_sdk/' + solution.ld_script_component.name + '/' + solution.ld_script_component.pack.hw_info.ld_script
                # text = pack.dumps()
                # print(text)
                # copy and set ld file
                ld_script_name = os.path.basename(solution.ld_script)
                pack.hw_info.ld_script = ld_script_name
                if os.path.dirname(solution.ld_script) != solution.solution_component.path:
                    shutil.copy2(solution.ld_script, pack.hw_info.ld_script)
                put_string("Rebuilding package.yaml")
                pack.defconfig.update(solution.org_defines)
                pack.save(os.path.join(solution.solution_component.path, 'package.yaml'))
                # Makefile
                put_string("Rebuilding Makefile")
                self.rebuild_makefile(solution.solution_component.path, 'Makefile')
                # gitignore
                put_string("Rebuilding .gitignore")
                self.rebuild_gitignore(solution.solution_component.path, '.gitignore')
                # .aos
                if not os.path.exists(os.path.join(os.getcwd(), '.aos')):
                    conf = Configure()
                    shutil.copy2(os.path.join(conf.yoc_path, ".aos"), os.getcwd())

                put_string("Generate %s SDK successful!" % solution.solution_component.name)
                return 0
        else:
            put_string("No solution found in current directory!")
        return 1

    def rebuild_makefile(self, path, filename):
        filename2 = filename + '.new'
        filepath = os.path.join(path, filename)
        filepath2 = os.path.join(path, filename2)
        if os.path.isfile(filepath):
            with codecs.open(filepath, 'r', 'UTF-8') as f:
                with codecs.open(filepath2, 'w', 'UTF-8') as fn:
                    for line in f.readlines():
                        if "rm" in line and "aos_sdk" in line:
                            fn.write("\t%s" % '$(CPRE) rm -rf out aos.elf aos_sdk generated/*.hex generated/*.zip\n')
                        else:
                            fn.write(line)
                    shutil.move(filepath2, filepath)

    def rebuild_gitignore(self, path, filename):
        filename2 = filename + '.new'
        filepath = os.path.join(path, filename)
        filepath2 = os.path.join(path, filename2)
        if os.path.isfile(filepath):
            with codecs.open(filepath, 'r', 'UTF-8') as f:
                with codecs.open(filepath2, 'w', 'UTF-8') as fn:
                    for line in f.readlines():
                        if "aos_sdk" not in line and "generated" not in line:
                            fn.write(line)
                    shutil.move(filepath2, filepath)





# -*- coding:utf-8 -*-
#
# Copyright (C) 2019-2020 Alibaba Group Holding Limited


import os
import sys
import shutil
import git
import time

from .log import *
from .tools import put_string


class simpleProgressBar(git.RemoteProgress):
    def update(self, op_code, cur_count, max_count=None, message=''):
        # text = "%3d%% (%d/%d)" % (cur_count/(max_count or 100.0), cur_count, max_count)
        sys.stdout.write(self._cur_line)
        sys.stdout.flush()
        if op_code & self.END:
            sys.stdout.write('\n')
        else:
            sys.stdout.write('\r')


class pullProgressBar(git.RemoteProgress):
    def update(self, op_code, cur_count, max_count=None, message=''):
        sys.stdout.write(self._cur_line)
        sys.stdout.flush()
        if op_code & self.END:
            sys.stdout.write('\n')
        else:
            sys.stdout.write('\r')


class GitRepo:
    def __init__(self, path, repo_url=None):
        self.repo_url = repo_url
        self.path = path
        git_path = os.path.join(path, '.git')

        if not os.path.exists(git_path):  # 如果未下载，则 git clone 下来
            self.repo = git.Repo.init(path)
        else:
            try:
                self.repo = git.Repo(path)
            except Exception as ex:
                self.repo = git.Repo.init(path)

        if repo_url:
            self.set_remote(repo_url)

    def set_remote(self, repo_url):
        try:
            origin = self.repo.remote()
            origin.set_url(repo_url)
        except Exception as ex:
            origin = self.repo.create_remote(name='origin', url=repo_url)
            try:
                origin.fetch()
            except Exception as ex:
                info = str(ex)
                if info.find("Please make sure you have the correct access rights") > 0:
                    put_string("%s" % origin.url)
                    put_string("Please make sure you have the correct access rights and the repository exists")

    def pull(self, version='', progress=pullProgressBar()):
        try:
            if not version:
                version = self.repo.active_branch

            origin = self.repo.remote()

            if version not in self.repo.heads:
                if version not in origin.refs:
                    origin.fetch(progress=progress)
                if version in origin.refs:
                    branch = self.repo.create_head(version, origin.refs[version])
                    branch.set_tracking_branch(origin.refs[version])
                    branch.checkout()
                elif version in self.repo.tags:
                    git = self.repo.git
                    git.checkout(version, b=version)         # create a new branch, name same as tag
            else:
                origin.pull(version)
                self.repo.heads[version].set_tracking_branch(origin.refs[version])

        except Exception as ex:
            info = str(ex)
            if info.find('Please make sure you have the correct access rights') > 0:
                put_string("%s" % origin.url)
                put_string("Please make sure you have the correct access rights and the repository exists.")
            # put_string("\nPull %s occur error:(%s)" % (origin.url, str(ex)), level='warning')
            pass

    def fetch(self, remote="origin"):
        try:
            remote = self.repo.remote(remote)
        except ValueError:
            msg = "Remote {remote} does not exist on repo {repo}".format(
                remote=remote,
                repo=self.repo.repo.working_dir
            )
            logger.error(msg)
        try:
            remote.fetch(progress=pullProgressBar())
        except git.GitCommandError as ex:
            info = str(ex)
            if info.find('Please make sure you have the correct access rights') > 0:
                put_string("%s" % remote.url)
                put_string("Please make sure you have the correct access rights and the repository exists.")
                # print("Use: `yoc addkey`")


    def import_path(self, path, version):
        files = os.listdir(self.repo.working_dir)
        for f in files:
            if f != '.git':
                fn = os.path.join(self.repo.working_dir, f)
                if os.path.isdir(fn):
                    shutil.rmtree(fn)
                else:
                    os.remove(fn)

        for dirpath, _, filenames in os.walk(path):
            if dirpath.find(os.path.join(path, '.git')) < 0:
                for f in filenames:
                    p1 = os.path.join(dirpath, f)
                    p2 = os.path.relpath(p1, path)
                    p2 = os.path.join(self.repo.working_dir, p2)
                    try:
                        p = os.path.dirname(p2)
                        os.makedirs(p)
                    except:
                        pass

                    shutil.copy2(p1, p2)

        if self.repo.is_dirty(untracked_files=True):
            self.repo.git.add(self.repo.untracked_files)
            self.repo.git.commit('-m', 'init version', '-a')

            branch = self.repo.create_head(version)
            branch.checkout()

            self.repo.git.push(
                "--set-upstream", self.repo.remotes.origin, self.repo.head.ref)

    def GetRemoteBranches(self, remote='origin'):
        br_arr = []
        try:
            branches = self.repo.remote().refs
            for b in branches:
                if remote and remote in b.name:
                    br_arr.append(''.join(b.name.split('/')[1:]))
                else:
                    br_arr.append(b.name)
        except:
            pass
        return br_arr

    def GetRemoteTags(self, remote=''):
        origin = self.repo.remote()
        origin.fetch("--tags")
        return self.repo.tags

    def CheckoutBranch(self, version):
        git = self.repo.git
        try:
            origin = self.repo.remote()
            if version not in origin.refs:
                git.checkout('HEAD', b=version)
            else:
                git.checkout(version)
        except Exception as ex:
            put_string(str(ex))
            pass
        self.pull(version)

    def GetActiveBranch(self):
        try:
            return self.repo.active_branch.name
        except TypeError:
            # {TypeError}HEAD is a detached symbolic reference as it points to '0fa1f2a7d9'
            return None

    def sync(self):
        name = os.path.basename(self.path)
        if self.repo.is_dirty(untracked_files=False):
            put_string("There are modified files in %s, please commit or stash first." % name)
            return
        self.pull()

    def status(self):
        name = os.path.basename(self.path)
        changed = [ item.a_path for item in self.repo.index.diff(None) ]
        untracked_files = self.repo.untracked_files
        if len(changed) > 0 or len(untracked_files) > 0:
            put_string("%s: on branch %s" % (name, self.repo.active_branch))
        for m in changed:
            put_string("        modified:   %s" % m)
        for uf in untracked_files:
            put_string("        %s" % uf)
        # git = self.repo.git
        # statusinfo = git.status()
        # if statusinfo.find("Changes not staged for commit:") > 0 or statusinfo.find("Untracked files:") > 0:
        #     put_string("%s:" % name)
        #     put_string("%s" % statusinfo)

    def gitlog(self):
        name = os.path.basename(self.path)
        put_string("%s:" % name, color='magenta')
        headcommit = self.repo.head.commit
        put_string("    commit %s" % headcommit.hexsha)
        put_string("    Author: %s <%s>" % (headcommit.author.name, headcommit.author.email))
        put_string("    Date:   %s" % time.strftime("%a %b %d %H:%M:%S %Y", time.localtime(headcommit.authored_date)))
        # put_string(headcommit.author_tz_offset)
        # put_string(headcommit.committer.name)
        # put_string(headcommit.committed_date)
        put_string("")
        put_string("        %s" % headcommit.message)

    def commit(self, message=''):
        name = os.path.basename(self.path)
        if self.repo.is_dirty(untracked_files=True):
            try:
                index = self.repo.index
                changed = [ item.a_path for item in self.repo.index.diff(None) ]
                untracked_files = self.repo.untracked_files
                files = []
                for a in changed:
                    files.append(a)
                for a in untracked_files:
                    files.append(a)
                index.add(files)
                index.commit(message)
                remote = self.repo.remote()
                remote.push()
                put_string("Commit [%-24s] ok." % name)
            except Exception as ex:
                put_string("Commit [%-24s] failed(%s)." % (name, str(ex)))

    def delete_branch(self, branch, is_remote=False):
        git = self.repo.git
        try:
            git.branch('-D', branch)
        except Exception as ex:
            # put_string("Delete branch:%s error(%s)." % (branch, str(ex)))
            if not is_remote:
                return str(ex)
        if is_remote:
            try:
                git.push('origin', ':'+branch)
            except Exception as ex:
                return str(ex)
        return None


# -*- coding:utf-8 -*-
#
# Copyright (C) 2019-2020 Alibaba Group Holding Limited


from __future__ import print_function
import os
import sys
import shutil
import chardet

import warnings
warnings.filterwarnings("ignore", category=SyntaxWarning)

try:
    try:
        import SCons.Script as SCons
    except:
        import scons
        for path in scons.__path__:
            sys.path.append(path)
            import SCons.Script as SCons
except:
    scons_path = []
    for path in sys.path:
        if os.path.isdir(os.path.join(path, "scons")):
            scons_path.append(os.path.join(path, "scons"))
    if scons_path:
        sys.path += scons_path
        import SCons.Script as SCons

from .log import logger
from .toolchain import *
from .qemu import *


# cpu models defines
CkcoreCPU = ['c807', 'c807f', 'c810', 'c810t', 'c810tv', 'c810v', 'c860', 'c860v', 'ck800', 'ck801', 'ck801t', 'ck802', 'ck802j', 'ck802t', 'ck803', 'ck803e', 'ck803ef', 'ck803efh', 'ck803efhr1', 'ck803efhr2', 'ck803efhr3', 'ck803efht', 'ck803efhtr1', 'ck803efhtr2', 'ck803efhtr3', 'ck803efr1', 'ck803efr2', 'ck803efr3', 'ck803eft', 'ck803eftr1', 'ck803eftr2', 'ck803eftr3', 'ck803eh', 'ck803ehr1', 'ck803ehr2', 'ck803ehr3', 'ck803eht', 'ck803ehtr1', 'ck803ehtr2', 'ck803ehtr3', 'ck803er1', 'ck803er2', 'ck803er3', 'ck803et', 'ck803etr1', 'ck803etr2', 'ck803etr3', 'ck803f', 'ck803fh', 'ck803fhr1', 'ck803fhr2', 'ck803fhr3', 'ck803fr1', 'ck803fr2', 'ck803fr3', 'ck803ft', 'ck803ftr1', 'ck803ftr2', 'ck803ftr3', 'ck803h', 'ck803hr1', 'ck803hr2', 'ck803hr3', 'ck803ht', 'ck803htr1', 'ck803htr2', 'ck803htr3', 'ck803r1', 'ck803r2', 'ck803r3', 'ck803s', 'ck803se', 'ck803sef', 'ck803seft', 'ck803sf', 'ck803st', 'ck803t', 'ck803tr1', 'ck803tr2', 'ck803tr3', 'ck804', 'ck804e', 'ck804ef', 'ck804efh', 'ck804efht', 'ck804eft', 'ck804eh', 'ck804eht', 'ck804et', 'ck804f', 'ck804fh', 'ck804ft', 'ck804h', 'ck804ht', 'ck804t', 'ck805', 'ck805e', 'ck805ef', 'ck805eft', 'ck805et', 'ck805f', 'ck805ft', 'ck805t', 'ck807', 'ck807e', 'ck807ef', 'ck807f', 'ck810', 'ck810e', 'ck810ef', 'ck810eft', 'ck810et', 'ck810f', 'ck810ft', 'ck810ftv', 'ck810fv', 'ck810t', 'ck810tv', 'ck810v', 'ck860', 'ck860f', 'ck860fv', 'ck860v', 'e801', 'e802', 'e802t', 'e803', 'e803t', 'e804d', 'e804df', 'e804dft', 'e804dt', 'e804f', 'e804ft', 'i805', 'i805f', 'r807', 'r807f', 's802', 's802t', 's803', 's803t']
RiscvCPU = ['rv32emc', 'rv32ec', 'rv32i', 'rv32iac', 'rv32im', 'rv32imac', 'rv32imafc', 'rv64imac', 'rv64imacxcki', 'rv64imafdc', 'e902', 'e902m', 'e902t', 'e902mt', 'e906', 'e906f', 'e906fd', 'c906', 'c906fd', 'c906v', 'c910', 'c910v']
ArmCPU = ['arm1020e', 'arm1020t', 'arm1022e', 'arm1026ej-s', 'arm10e', 'arm10tdmi', 'arm1136j-s', 'arm1136jf-s', 'arm1156t2-s',
                            'arm1156t2f-s', 'arm1176jz-s', 'arm1176jzf-s', 'arm2', 'arm250', 'arm3', 'arm6', 'arm60', 'arm600', 'arm610', 'arm620',
                            'arm7', 'arm70', 'arm700', 'arm700i', 'arm710', 'arm7100', 'arm710c', 'arm710t', 'arm720', 'arm720t', 'arm740t', 'arm7500',
                            'arm7500fe', 'arm7d', 'arm7di', 'arm7dm', 'arm7dmi', 'arm7m', 'arm7tdmi', 'arm7tdmi-s', 'arm8', 'arm810', 'arm9', 'arm920',
                            'arm920t', 'arm922t', 'arm926ej-s', 'arm940t', 'arm946e-s', 'arm966e-s', 'arm968e-s', 'arm9e', 'arm9tdmi', 'cortex-a12',
                            'cortex-a15', 'cortex-a17', 'cortex-a32', 'cortex-a35', 'cortex-a5','cortex-a53', 'cortex-a57', 'cortex-a7', 'cortex-a72',
                            'cortex-a73', 'cortex-a8', 'cortex-a9', 'cortex-m0', 'cortex-m0.small-multiply', 'cortex-m0plus', 'cortex-m0plus.small-multiply',
                            'cortex-m1', 'cortex-m1.small-multiply', 'cortex-m23', 'cortex-m3', 'cortex-m33', 'cortex-m33+nodsp', 'cortex-m4', 'cortex-m7',
                            'cortex-r4', 'cortex-r4f', 'cortex-r5', 'cortex-r52', 'cortex-r7', 'cortex-r8']

class Builder(object):
    def __init__(self, solution):
        self.toolchain_path = ''
        self.qemu_path = ''
        self.PREFIX = 'csky-abiv2-elf'
        if solution.toolchain_prefix:
            self.PREFIX = solution.toolchain_prefix
        self.SIZE = lambda: self.PREFIX + '-size'
        self.OBJDUMP = lambda: self.PREFIX + '-objdump'
        self.OBJCOPY = lambda: self.PREFIX + '-objcopy'
        self.STRIP = lambda: self.PREFIX + '-strip'
        self.AS = lambda: self.PREFIX + '-gcc'
        self.CC = lambda: self.PREFIX + '-gcc'
        self.CXX = lambda: self.PREFIX + '-g++'
        self.AR = lambda: self.PREFIX + '-ar'
        self.LINK = lambda: self.PREFIX + '-g++'
        self.RANLIB = lambda: self.PREFIX + '-ranlib'
        self.postimage_scripts = []

        self.solution = solution

        self.env = SCons.Environment(tools=['as', 'gcc', 'g++', 'ar', 'gnulink', 'objcopy', 'objdump', 'product'],
                                     toolpath=[os.path.dirname(
                                         __file__) + '/site_tools'],
                                     AS=self.AS(),
                                     CC=self.CC(),
                                     CXX=self.CXX(),
                                     AR=self.AR(),
                                     LINK=self.CXX(),
                                     OBJCOPY=self.OBJCOPY(),
                                     OBJDUMP=self.OBJDUMP(),
                                     RANLIB=self.RANLIB(),
                                     PROGSUFFIX='.elf',
                                     ARFLAGS='-rc',
                                     )

        # self.env.Decider(decide_if_changed)
        self.env.Decider('timestamp-newer')
        # self.env.Decider('make')
        # self.env.Decider('MD5')

        self.env.PrependENVPath('TERM', "xterm-256color")
        self.env.PrependENVPath('PATH', os.getenv('PATH'))

        if SCons.GetOption('verbose'):
            self.env.Replace(
                ARCOMSTR='AR $TARGET',
                ASCOMSTR='AS $TARGET',
                ASPPCOMSTR='AS $TARGET',
                CCCOMSTR='CC $TARGET',
                CXXCOMSTR='CXX $TARGET',
                LINKCOMSTR = 'LINK $TARGET',
                RANLIBCOMSTR = 'RANLIB $TARGET',
                INSTALLSTR='INSTALL $TARGET',
                BINCOMSTR="Generating $TARGET",
            )

        self.set_cpu(self.solution.cpu_name)
        if self.solution.LINKFLAGS:
            linkflags = self.solution.LINKFLAGS
        else:
            mapfile = "%s@%s.map" % (self.solution.solution_component.name, self.solution.solution_component.hw_info.board_name)
            mapfile = os.path.join("out", mapfile)
            if self.solution.cpu_name.lower().startswith('ck'):
                linkflags = ['-lm', '-Wl,-ckmap="%s"' % mapfile, '-Wl,-zmax-page-size=1024']
            else:
                linkflags = ['-lm', '-Wl,-Map="%s"' % mapfile, '-Wl,-zmax-page-size=1024']
        self.env.AppendUnique(
            ASFLAGS=self.solution.ASFLAGS,
            CFLAGS=self.solution.CFLAGS,
            CXXFLAGS=self.solution.CXXFLAGS,
            LINKFLAGS=linkflags,
        )

        self.env.Replace(AS=self.AS(),
                        CC=self.CC(),
                        CXX=self.CXX(),
                        AR=self.AR(),
                        LINK=self.CXX(),
                        OBJCOPY=self.OBJCOPY(),
                        OBJDUMP=self.OBJDUMP(),
                        RANLIB=self.RANLIB())

    def set_cpu(self, cpu):
        flags = ['-MP', '-MMD', '-g', '-Os', '-Wno-main']
        self.CPU = cpu.lower()
        if self.CPU in CkcoreCPU:
            if not self.PREFIX:
                self.PREFIX = 'csky-abiv2-elf'
            flags.append('-mcpu=' + self.CPU)
            if 'f' in self.CPU:
                flags.append('-mhard-float')
            if self.CPU == 'ck803ef':
                flags.append('-mhigh-registers')
                flags.append('-mdsp')
        elif self.CPU in RiscvCPU:
            if not self.PREFIX:
                self.PREFIX = 'riscv64-unknown-elf'
            elif not self.PREFIX.startswith('riscv'):
                self.PREFIX = 'riscv64-unknown-elf'
            if self.CPU == 'rv32emc':
                flags.append('-march=' + self.CPU)
                flags.append('-mabi=ilp32e')
            if self.CPU == 'rv32ec':
                flags.append('-march=' + self.CPU)
                flags.append('-mabi=ilp32e')
            if self.CPU == 'rv32i':
                flags.append('-march=' + self.CPU)
                flags.append('-mabi=ilp32')
                flags.append('-mcmodel=medlow')
            if self.CPU == 'rv32iac':
                flags.append('-march=' + self.CPU)
                flags.append('-mabi=ilp32')
                flags.append('-mcmodel=medlow')
            if self.CPU == 'rv32im':
                flags.append('-march=' + self.CPU)
                flags.append('-mabi=ilp32')
                flags.append('-mcmodel=medlow')
            if self.CPU == 'rv32imac':
                flags.append('-march=' + self.CPU)
                flags.append('-mabi=ilp32')
                flags.append('-mcmodel=medlow')
            if self.CPU == 'rv32imafc':
                flags.append('-march=' + self.CPU)
                flags.append('-mabi=ilp32f')
                flags.append('-mcmodel=medlow')
            if self.CPU == 'rv64imac':
                flags.append('-march=' + self.CPU)
                flags.append('-mabi=lp64')
                flags.append('-mcmodel=medany')
            if self.CPU == 'rv64imacxcki':
                flags.append('-march=' + self.CPU)
                flags.append('-mabi=lp64')
                flags.append('-mcmodel=medany')
            if self.CPU == 'rv64imafdc':
                flags.append('-march=' + self.CPU)
                flags.append('-mabi=lp64d')
                flags.append('-mcmodel=medany')
            if self.CPU == 'e902':
                flags.append('-march=rv32ecxtheadse')
                flags.append('-mabi=ilp32e')
            if self.CPU == 'e902m':
                flags.append('-march=rv32emcxtheadse')
                flags.append('-mabi=ilp32e')
            if self.CPU == 'e902t':
                flags.append('-march=rv32ecxtheadse')
                flags.append('-mabi=ilp32e')
            if self.CPU == 'e902mt':
                flags.append('-march=rv32emcxtheadse')
                flags.append('-mabi=ilp32e')
            if self.CPU == 'e906':
                flags.append('-march=rv32imacxthead')
                flags.append('-mabi=ilp32')
                flags.append('-mcmodel=medlow')
            if self.CPU == 'e906f':
                flags.append('-march=rv32imafcxthead')
                flags.append('-mabi=ilp32f')
                flags.append('-mcmodel=medlow')
            if self.CPU == 'e906fd':
                flags.append('-march=rv32imafdcxthead')
                flags.append('-mabi=ilp32d')
                flags.append('-mcmodel=medlow')
            if self.CPU == 'c906':
                flags.append('-march=rv64imacxtheadc')
                flags.append('-mabi=lp64')
                flags.append('-mcmodel=medlow')
                flags.append('-mtune=c906')
            if self.CPU == 'c906fd':
                flags.append('-march=rv64imafdcvxtheadc')
                flags.append('-mabi=lp64d')
                flags.append('-mcmodel=medany')
                flags.append('-mtune=c906')
            if self.CPU == 'c906v':
                flags.append('-march=rv64imafdcvxtheadc')
                flags.append('-mabi=lp64v')
                flags.append('-mcmodel=medany')
                flags.append('-mtune=c906')
            if self.CPU == 'c910':
                flags.append('-march=rv64imafdcvxtheadc')
                flags.append('-mabi=lp64d')
                flags.append('-mcmodel=medany')
                flags.append('-mtune=c910')
            if self.CPU == 'c910v':
                flags.append('-march=rv64imafdcvxtheadc')
                flags.append('-mabi=lp64v')
                flags.append('-mcmodel=medany')
                flags.append('-mtune=c910')
        elif self.CPU in ArmCPU:
            if not self.PREFIX:
                self.PREFIX = 'arm-none-eabi'
            elif not self.PREFIX.startswith('arm'):
                self.PREFIX = 'arm-none-eabi'
            flags.append('-mcpu=' + self.CPU)
        else:
            logger.error(
                'error cpu `%s`, please make sure your cpu mode' % self.CPU)
            exit(-1)

        self.env.AppendUnique(
            ASFLAGS=flags, CFLAGS=flags,
            CXXFLAGS=flags, LINKFLAGS=flags
        )

    def clone_component(self, component):
        def var_convert(defs):
            if type(defs) == dict:
                vars = {}
                for k, v in defs.items():
                    if type(v) == str:
                        vars[k] = '\\"' + v + '\\"'
                    else:
                        vars[k] = v
                return vars
            else:
                return defs

        env = self.env.Clone()

        if component.build_config.cflag:
            env.AppendUnique(CFLAGS=component.build_config.cflag.split())
        if component.build_config.cxxflag:
            env.AppendUnique(CXXFLAGS=component.build_config.cxxflag.split())
        if component.build_config.asmflag:
            env.AppendUnique(ASFLAGS=component.build_config.asmflag.split())

        env.AppendUnique(CPPPATH=component.build_config.internal_include_updated)
        env.AppendUnique(CPPPATH=self.solution.global_includes)
        env.AppendUnique(CPPDEFINES=var_convert(self.solution.defines))
        env.AppendUnique(CPPDEFINES=var_convert(component.build_config.define))

        # when dummy, use qemu platform
        if self.solution.variables.get('vendor') == 'dummy':
            if self.qemu_path == '':
                qemu = QemuYoC()
                path = qemu.check_qemu(self.solution.variables.get('arch'))
                if path:
                    self.qemu_path = os.path.dirname(path)
            if self.qemu_path:
                env.PrependENVPath('PATH', self.qemu_path)
            else:
                put_string("Not found qemu for %s, please check it." % self.solution.variables.get('arch'))
                exit(-1)

        if self.toolchain_path == '':
            tool = ToolchainYoC()
            path = tool.check_toolchain(self.PREFIX)
            if path:
                self.toolchain_path = os.path.dirname(path)

        if self.toolchain_path:
            env.PrependENVPath('PATH', self.toolchain_path)
        else:
            put_string("Not found toolchain: `%s`, please check it." % self.PREFIX)
            exit(-1)

        return env

    def build_component(self, component):
        env = self.clone_component(component)

        sources = []
        for fn in component.source_files_updated:
            f_list = env.Glob(fn)
            if f_list:
                for f in f_list:
                    if f not in sources:
                        sources.append(f)

        job = env.StaticLibrary(os.path.join(
            self.solution.lib_path, component.name), sources)

        def convert_obj_to_string(objs):
            result = ""
            if type(objs) == dict:
                for k, v in objs.items():
                    if type(v) == str:
                        result += ('#-D%s=\\"%s\\"') % (k, v)
                    elif type(v) == int:
                        result += ('#-D%s=%d') % (k, v)
            elif type(objs) == list:
                for obj in objs:
                    if type(obj) == dict:
                        for k, v in obj.items():
                            if type(v) == str:
                                result += ('#-D%s=\\"%s\\"') % (k, v)
                            elif type(v) == int:
                                result += ('#-D%s=%d') % (k, v)
                    elif type(obj) == str:
                        result += ('#-D%s') % (obj)
            return result

        def convert_macro_to_string(global_defines, local_defines):
            macros_str = convert_obj_to_string(global_defines)
            macros_str += convert_obj_to_string(local_defines)
            macros_str = macros_str.replace(' ', '#')
            macros_str = macros_str.strip('#')
            return macros_str

        def convert_list_to_string(strlist):
            to_str = ""
            for item in strlist:
                to_str += '#\\"%s\\"' % item.replace(' ', '#')
            to_str = to_str.strip('#')
            return to_str

        def run_external_script(script_file, sys_params):
            params_list = ""
            if script_file.endswith(".py"):
                params_list = "python "
            params_list += "\"%s\" %s" % (script_file, sys_params)
            if not SCons.GetOption('verbose'):
                put_string("script is %s" % params_list)
            script_process = subprocess.Popen(params_list, shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
            lines = script_process.stdout.readlines()
            script_result = False
            for text in lines:
                try:
                    encode = chardet.detect(text)
                    if not encode["encoding"]:
                        encode["encoding"] = 'utf-8'
                    text = text.decode(encode["encoding"]).strip()
                except UnicodeDecodeError:
                    text = " "
                    pass
                if text.find("run external script success") >= 0:
                    script_result = True
            if not script_result:
                put_string("Run script '%s' failed" % script_file)
            else:
                put_string("Run script '%s' success" % script_file)
            if not script_result or not SCons.GetOption('verbose'):
                put_string("***********script log is as below:*******************")
                for line in lines:
                    try:
                        encode = chardet.detect(line)
                        if not encode["encoding"]:
                            encode["encoding"] = 'utf-8'
                        line = line.decode(encode["encoding"]).strip()
                    except UnicodeDecodeError:
                        line = " "
                        pass
                    put_string(line)
                put_string("*****************************************************")
            return script_result

        def run_postbuild_script(target, source, env):
            sys_params = " --toolchain=\"%s\" " % os.path.join(self.toolchain_path, self.PREFIX)
            if self.solution.CFLAGS:
                sys_params += " --cflag=%s " % ('#').join(self.solution.CFLAGS)
            if self.solution.CXXFLAGS:
                sys_params += " --cxxflag=%s " % ('#').join(self.solution.CXXFLAGS)
            if self.solution.ASFLAGS:
                sys_params += " --asmflag=%s " % ('#').join(self.solution.ASFLAGS)
            if self.solution.LINKFLAGS:
                sys_params += " --ldflag=%s " % ('#').join(self.solution.LINKFLAGS)
            if self.solution.cpu_name:
                sys_params += " --cpu=%s " % self.solution.cpu_name.lower()
            sys_params += " --lib=\"%s\" " % os.path.abspath("%s" % target[0])
            sys_params += " --solution_dir=\"%s\" " % self.solution.solution_component.path
            sys_params += " --comp_name=%s " % component.name
            sys_params += " --macro_list=%s" % convert_macro_to_string(self.solution.defines, component.build_config.define)
            sys_params += " --global_inc=%s" % convert_list_to_string(self.solution.global_includes)
            run_external_script(component.build_config.postbuild_script, sys_params)

        def run_postimage_script(target, source, env):
            sys_params = " --target=\"%s\" " % os.path.abspath("%s" % target[0])
            for script in self.postimage_scripts:
                run_external_script(script, sys_params)

        def run_memorymap_script(target, source, env):
            map_file = ""
            for item in env['LINKFLAGS']:
                if "Map=" in item:
                    pattern = re.compile(r'-Map=\"(.*)\"')
                    match = pattern.search(item)
                    if match:
                        map_file = os.path.abspath("%s" % match.group(1))
                        get_mem_info(map_file)
                        break

        if component.build_config.prebuild_script:
            if SCons.SCons.SConf.build_type != "clean":
                sys_params = " --toolchain=\"%s\" " % os.path.join(self.toolchain_path, self.PREFIX)
                if self.solution.CFLAGS:
                    sys_params += " --cflag=%s " % ('#').join(self.solution.CFLAGS)
                if self.solution.CXXFLAGS:
                    sys_params += " --cxxflag=%s " % ('#').join(self.solution.CXXFLAGS)
                if self.solution.ASFLAGS:
                    sys_params += " --asmflag=%s " % ('#').join(self.solution.ASFLAGS)
                if self.solution.LINKFLAGS:
                    sys_params += " --ldflag=%s " % ('#').join(self.solution.LINKFLAGS)
                if self.solution.cpu_name:
                    sys_params += " --cpu=%s " % self.solution.cpu_name.lower()
                sys_params += " --lib=\"%s\" " % os.path.join(self.solution.lib_path, "lib%s.a" % component.name)
                sys_params += " --solution_dir=\"%s\" " % self.solution.solution_component.path
                sys_params += " --comp_name=%s " % component.name
                sys_params += " --macro_list=%s" % convert_macro_to_string(self.solution.defines, component.build_config.define)
                sys_params += " --global_inc=%s" % convert_list_to_string(self.solution.global_includes)
                run_external_script(component.build_config.prebuild_script, sys_params)

        if component.build_config.postbuild_script:
            env.AddPostAction(job, env.Action(run_postbuild_script))

        if component.build_config.postimage_script:
            self.postimage_scripts.append(component.build_config.postimage_script)

        env.Default(job)

        if component.type == 'solution':
            # for libc, it should be -lc, not -llibc
            linkflags = ' -Wl,--whole-archive'
            libs = self.solution.libs
            libs.append(component.name)
            for lib in libs:
                if lib.startswith('lib'):
                    linkflags += ' -l' + lib[3:]
                else:
                    linkflags += ' -l' + lib
            linkflags += ' -Wl,--no-whole-archive'
            linkflags += ' -nostartfiles -Wl,--gc-sections'
            linkflags += ' -T "' + self.solution.ld_script + '"'
            cname = 'aos'  # component.name
            env.AppendUnique(LINKFLAGS=linkflags.split())
            env.AppendUnique(LIBPATH=self.solution.libpath)
            job = env.Program(target=cname, source=[])

            # add recompiled file check.
            env.Depends(job, self.solution.depend_libs)
            env.Depends(job, self.solution.ld_script)
            if self.postimage_scripts and not env['OBJCOPY_FILE']:
                env.AddPostAction(job, env.Action(run_postimage_script))
            env.Default(job)

            jobs = []
            dirname = os.path.dirname(env.GetBuildPath("output_xxxd"))
            if env['ELF_FILE']:
                output = os.path.join(dirname, env['ELF_FILE'])
                jj = env.InstallAs(output, job[0])
                jobs.append(jj)

            if env['OBJCOPY_FILE']:
                output = os.path.join(dirname, env['OBJCOPY_FILE'])
                jj = env.Binary(source=job[0], target=output)
                if self.postimage_scripts:
                    env.AddPostAction(jj, env.Action(run_postimage_script))
                jobs.append(jj)

            if env['OBJDUMP_FILE']:
                output = os.path.join(dirname, env['OBJDUMP_FILE'])
                jj = env.Dump(source=job[0], target=output)
                jobs.append(jj)

            env.AddPostAction(job, env.Action(run_memorymap_script))
            env.Default(jobs)

    def build_image(self, elf=None, objcopy=None, objdump=None, product=None):
        component = self.solution.solution_component
        env = self.clone_component(component)

        source_name = os.path.join('out', component.name, 'aos.elf')
        if elf and os.path.isfile(source_name):
            shutil.copy2(source_name, elf)


        if objcopy:
            job1 = env.Binary(source=source_name, target=objcopy)
            env.Default(job1)

        if objdump:
            job2 = env.Dump(source=source_name, target=objdump)
            env.Default(job2)
        if product:
            job3 = env.Zip(source='generated/data/config.yaml',
                        target="generated/images.zip", PATH='generated/data')
            job4 = env.Hex(source='generated/images.zip', PATH='generated')
            env.Default(job3)
            env.Default(job4)


def decide_if_changed(dependency, target, prev_ni, repo_node=None):
    # put_string(dependency, prev_ni)
    if not prev_ni:
        return True
    if dependency.get_timestamp() != prev_ni.timestamp:
        return True

    return False



# -*- coding:utf-8 -*-
#
# Copyright (C) 2019-2020 Alibaba Group Holding Limited


from __future__ import print_function

from aostools import *
from .variable import *

class Config(Variable):
    pass



# -*- coding:utf-8 -*-
#
# Copyright (C) 2019-2020 Alibaba Group Holding Limited


from __future__ import print_function
import os
import sys
import time
import hashlib
import zipfile
import codecs
import json
import re
import platform
import subprocess
import chardet
import glob

try:
    from urlparse import urlparse
    import urllib
    import httplib as http
    urlretrieve = urllib.urlretrieve
    import urllib2 as urllib
except:
    from urllib.parse import urlparse
    import urllib.request
    import http.client as http
    urlretrieve = urllib.request.urlretrieve


try:
    import yaml
except:
    put_string("\n\nNot found pyyaml, please install: \nsudo pip install pyyaml")
    sys.exit(-1)

def is_contain_chinese(check_str):
    if sys.version_info.major == 2:
        if type(check_str) == str:
            check_str = check_str.decode('utf-8')
    zhPattern = re.compile(u'[\u4e00-\u9fa5]+')
    match = zhPattern.search(check_str)
    if match:
        return True

def is_leagal_name(name):
    pattern = re.compile(u'^[_a-zA-Z]\w*$')
    match = pattern.search(name)
    if match:
        return True

def string_len(text):
    L = 0
    R = ''
    if sys.version_info.major == 2:
        if type(text) == str:
            text = text.decode('utf8')
    for c in text:
        if ord(c) >= 0x4E00 and ord(c) <= 0x9FA5:
            L += 2
        else:
            L += 1
    return L


def put_string(*args, **keys):
    for a in args:
        if sys.version_info.major == 2:
            if type(a) == unicode:
                a = a.encode('utf8')
        if 'key' in keys:
            key = keys['key']
            color_print(a, key=key, end=' ', ignorecase=True)
        else:
            print(a, end=' ')
    print()


def color_print(text, key=[], end='\n', ignorecase=False):
    idx = {}
    itext = text
    if ignorecase:
        itext = text.lower()
    for k in key:
        index = 0
        while True:
            s = itext.find(k, index)
            if s >= 0:
                e = s + len(k)
                need_del = []
                for a, b in idx.items():
                    if max(s, a) <= min(e, b):
                        s = min(s, a)
                        e = max(e, b)
                        need_del.append(a)
                for v in need_del:
                    del idx[v]
                idx[s] = e
                index = e
            else:
                break
    s = 0
    for v in list(sorted(idx)):
        e = v
        print(text[s: e], end='')
        print('\033[1;31m' + text[v: idx[v]] + '\033[0m', end='')
        s = idx[v]
    print(text[s: len(text)], end=end)


def yaml_load(filename):
    try:
        with codecs.open(filename, 'r', encoding='UTF-8') as fh:
            text = fh.read()
            return yaml.safe_load(text)
    except Exception as e:
        put_string("(%s) in file:%s" % (str(e), filename))
        exit(-1)


def home_path(path=''):
    return os.path.join(os.path.expanduser('~'), path)


def http2git(url):
    conn = urlparse(url)
    url = 'git@' + conn.netloc + ':' + conn.path[1:]
    return url


def MD5(str):
    hl = hashlib.md5()
    hl.update(str.encode(encoding='utf-8'))
    return hl.hexdigest()


def http_request(method, url, data=None, headers={}):
    conn = urlparse(url)

    if conn.scheme == "https":
        connection = http.HTTPSConnection(conn.netloc)
    else:
        connection = http.HTTPConnection(conn.netloc)
    # connection.debuglevel = 1

    connection.request(method=method, url=conn.path + '?' +
                       conn.query, body=data, headers=headers)
    response = connection.getresponse()
    return response.status, response.read(), response.msg



def http_get(url, path):
    conn = urlparse(url)

    if conn.scheme == "https":
        connection = http.HTTPSConnection(conn.netloc)
    else:
        connection = http.HTTPConnection(conn.netloc)

    connection.request('GET', conn.path)
    response = connection.getresponse()

    filename = os.path.join(path, os.path.basename(conn.path))

    try:
        with codecs.open(filename, 'wb', encoding='UTF-8') as f:
            f.write(response.read())
    except:
        pass

    return filename


def wget(url, out_file):
    start_time = time.time()

    def barProcess(blocknum, blocksize, totalsize):
        speed = (blocknum * blocksize) / (time.time() - start_time)
        # speed_str = " Speed: %.2f" % speed
        speed_str = " Speed: %sB/S         " % format_size(speed)
        recv_size = blocknum * blocksize

        # 设置下载进度条
        f = sys.stdout
        percent = float(recv_size) / totalsize
        if percent > 1:
            percent = 1
        percent_str = " %.2f%%" % (percent * 100)
        n = int(percent * 50)
        s = ('#' * n).ljust(50, '-')
        f.write(percent_str.ljust(9, ' ') + '[' + s + ']' + speed_str)
        f.flush()
        f.write('\r')

    def format_size(bytes):
        bytes = float(bytes)
        kb = bytes / 1024
        if kb >= 1024:
            M = kb / 1024
            if M >= 1024:
                G = M / 1024
                return "%.3fG" % (G)
            else:
                return "%.3fM" % (M)
        else:
            return "%.3fK" % (kb)

    return urlretrieve(url, out_file, barProcess)


# make_archive(base_name, format, root_dir=None, base_dir=None, verbose=0,dry_run=0, owner=None, group=None, logger=None)
def dfs_get_zip_file(input_path, result):
    files = os.listdir(input_path)
    for file in files:
        if os.path.isdir(input_path + '/' + file):
            dfs_get_zip_file(input_path + '/' + file, result)
        else:
            result.append(input_path + '/' + file)


def version_inc(v, x):
    l = len(v)
    num_start = -1
    for i in range(l - 1, -1, -1):
        if num_start == -1:
            if v[i:i + 1].isdigit():
                num_start = i + 1
        else:
            if not v[i:i + 1].isdigit():
                s = v[i + 1:num_start]
                v2 = v.replace(s, str(int(s) + x))
                return v2

    return v + '-0'


def zip_path(input_path, zipName):
    if os.path.isdir(input_path):
        base = os.path.dirname(zipName)
        try:
            os.makedirs(base)
        except:
            pass
        predir = input_path.rsplit('/', 1)[0]
        f = zipfile.ZipFile(zipName, 'w', zipfile.ZIP_DEFLATED)
        filelists = []
        dfs_get_zip_file(input_path, filelists)
        for file in filelists:
            suffix = os.path.splitext(file)[-1]
            if suffix != '.d' and suffix != '.o':
                f.write(file, file.split(predir)[1])
        f.close()

def unzip_path(output_path, zipName):
    if os.path.exists(output_path):
        if os.path.isfile(output_path):
            os.remove(output_path)
            os.makedirs(output_path)
    else:
        os.makedirs(output_path)
    
    file = os.path.abspath(zipName)
    if zipfile.is_zipfile(file):
        try:
            with zipfile.ZipFile(file, "r") as myzip:
                badcrc = myzip.testzip()
                if badcrc:
                    put_string("Bad CRC for file %s of the zip archive" % badcrc)
                myzip.extractall(output_path)
                return True
        except Exception as e:
            put_string("Failed to install from zip file, error: %s!" % format(e))
    else:
        put_string("%s is not zip file" % file)
    return False

def write_file(text, filename):
    contents = None

    try:
        with codecs.open(filename, 'r', encoding='UTF-8') as f:
            contents = f.read()
    except:
        pass

    if text == contents:
        return
    try:
        p = os.path.dirname(filename)
        try:
            os.makedirs(p)
        except:
            pass

        with codecs.open(filename, 'w', encoding='UTF-8') as f:
            f.write(text)
    except:
        put_string("Generate %s file failed." % filename)


def genSConcript(path):
    file_name = os.path.join(path, 'SConscript')
    text = '''Import('defconfig')\ndefconfig.library_yaml()\n'''
    write_file(text, file_name)
    return file_name


def genSConstruct(components, path):
    text = """#! /bin/env python

from aostools.make import Make

defconfig = Make()

Export('defconfig')

defconfig.build_components()
defconfig.program()
"""

    comp_list = ''
    for component in components:
        if component != '.':
            comp_list += '    "' + component.name + '",\n'

    text = text % comp_list

    script_file = os.path.join(path, 'SConstruct')
    write_file(text, script_file)


def genMakefile(path):
    text = """CPRE := @
ifeq ($(V),1)
CPRE :=
endif


.PHONY:startup
startup: all

all:
	$(CPRE) scons -j4
	@echo YoC SDK Done


.PHONY:clean
clean:
	$(CPRE) rm -rf aos_sdk
	$(CPRE) scons -c
	$(CPRE) find . -name "*.[od]" -delete

%:
	$(CPRE) scons --component="$@" -j4
"""

    script_file = os.path.join(path, 'Makefile')
    write_file(text, script_file)


def save_yoc_config(defines, filename):
    contents = ""

    try:
        with codecs.open(filename, 'r', encoding='UTF-8') as f:
            contents = f.read()
    except:
        pass

    text = '''/* don't edit, auto generated by tools/toolchain.py */\n
#ifndef __YOC_CONFIG_H__
#define __YOC_CONFIG_H__
#ifndef CONFIG_CPU\n\n'''
    for k, v in defines.items():
        if v in ['y', 'Y']:
            text += '#define %s 1\n' % k
        elif v in ['n', 'N']:
            text += '// #define %s 1\n' % k
        elif type(v) == int:
            text += '#define %s %d\n' % (k, v)
        else:
            text += '#define %s "%s"\n' % (k, v)

    text += '\n#endif\n#endif\n'

    if text == contents:
        return False

    write_file(text, filename)


def save_csi_config(defines, filename):
    text = '''/* don't edit, auto generated by tools/toolchain.py */

#ifndef __CSI_CONFIG_H__
#define __CSI_CONFIG_H__

#include <yoc_config.h>

#endif

'''

    write_file(text, filename)

def get_cmpt_path_by_type(type):
    if type == 'board':
        path = os.path.join('hardware', 'board')
    elif type == 'chip':
        path = os.path.join('hardware', 'chip')
    elif type == 'arch':
        path = os.path.join('hardware', 'arch')
    elif type == 'drv_core':
        path = os.path.join('components', 'drivers', 'core')
    elif type == 'drv_peripheral':
        path = os.path.join('components', 'drivers', 'peripheral')
    elif type == 'drv_external_device':
        path = os.path.join('components', 'drivers', 'external_device')
    elif type == 'kernel':
        path = 'kernel'
    elif type == 'solution':
        path = 'solutions'
    elif type == 'document':
        path = '.'
    else:
        # common and sdk
        path = 'components'
    return path

def get_cmpt_top_path_by_type(type):
    if type == 'board' or type == 'chip' or type == 'arch':
        path = 'hardware'
    elif type == 'drv_core' or type == 'drv_peripheral' or type == 'drv_external_device':
        path = 'components'
    elif type == 'kernel':
        path = 'kernel'
    elif type == 'solution':
        path = 'solutions'
    elif type == 'document':
        path = '.'
    else:
        # common and sdk
        path = 'components'
    return path

def get_type_by_cmpt_path(path):
    path = os.path.abspath(path)
    dirname = os.path.dirname(path)
    name = os.path.basename(dirname)
    if name in ['board', 'chip', 'arch', 'kernel']:
        comp_type = name
    elif name == 'core':
        comp_type = 'drv_core'
    elif name == 'peripheral':
        comp_type = 'drv_peripheral'
    elif name == 'external_device':
        comp_type = 'drv_external_device'
    elif name == 'solutions':
        comp_type = 'solution'
    else:
        comp_type = 'common'
    return comp_type

def get_host_os():
    host_os = platform.system()
    if host_os == 'Windows':
        host_os = 'Win32'
    elif host_os == 'Linux':
        if platform.machine().endswith('64'):
            bit = '64'
        else:
            bit = '32'
        host_os += bit
    elif host_os == 'Darwin':
        host_os = 'OSX'
    else:
        host_os = None
    return host_os

def find_latest_file(src):
    latest_modified_time = 0.0
    latest_file = ""
    for s in glob.iglob(src):
        modified_time = os.stat(s).st_mtime
        if modified_time > latest_modified_time:
            latest_modified_time = modified_time
            latest_file = s
    return latest_file

def get_elf_bin_file(filename):
    elf_file = ''
    bin_file = ''
    if not os.path.isfile(filename):
        return elf_file, bin_file
    try:
        with codecs.open(filename, 'r', encoding='UTF-8') as fh:
            lines = fh.readlines()
            elf_pattern = re.compile('elf\s*=\s*[\'\"](.+?)[\'\"]')
            bin_pattern = re.compile('objcopy\s*=\s*[\'\"](.+?)[\'\"]')
            for line in lines:
                if line.startswith('defconfig'):
                    match = elf_pattern.search(line)
                    if match:
                        elf_file = match.group(1)
                    match = bin_pattern.search(line)
                    if match:
                        bin_file = match.group(1)
                    break
            # if not specified, search in out directory
            if not elf_file:
                elf_file = find_latest_file(os.path.join("out", "*@*.elf"))
            if not bin_file:
                bin_file = find_latest_file(os.path.join("out", "*@*.bin"))
            return elf_file, bin_file
    except Exception as e:
        put_string("(%s) in file:%s" % (str(e), filename))
        exit(-1)

def which(program, extra_path=None):
    if get_host_os() == 'Win32' and program.endswith('.exe') == False:
        program += '.exe'

    def is_exe(fpath):
        return os.path.isfile(fpath) and os.access(fpath, os.X_OK)

    fpath, fname = os.path.split(program)
    if fpath:
        if is_exe(program):
            return program
    else:
        paths = os.environ["PATH"].split(os.pathsep)
        if extra_path:
            paths += extra_path.split(os.pathsep)

        for path in paths:
            exe_file = os.path.join(path, program)
            if is_exe(exe_file):
                return exe_file
    return None

def get_mem_info(map_file):
    total_ro = 0
    total_rw = 0
    map_lines = []
    with open(map_file, 'r') as f:
        s = f.read()
        pattern_str = 'Linker script and memory map([\s\S]+?)OUTPUT\(\S*%s\s' % os.path.basename(map_file).replace('.map','.elf')
        # find memory map (without discard and debug sections)
        mem_map_list = re.findall(pattern_str, s)
        mem_map = '' if not mem_map_list else mem_map_list[0]
        if not mem_map:
            print('Can\'t parse memory info, memory info get fail!')
            return

        mem_map = mem_map.replace('\r', '')
        modules = []
        for l in mem_map.split('\n'):
            m = re.search(r'0x\w+\s+0x\w+\s+.+?([^/\\]+\.[ao])\((.+\.o)\)', l)
            if m == None:
                continue
            modules.append(m.groups()[0])
        modules = list(set(modules))

        # find sections address - length in memory map
        # modules = list(set(item[0] for item in re.findall('0x\w+\s+0x\w+\s+.+?([^/\\\]+\.[ao])(\(.+\.o\))?\r?\n', mem_map)))
        modules.sort(key = lambda x : x.upper())

        for module in modules:
            module = module.replace('+', r'\+')

            # get module's sections's address and size
            sec_text    = map(lambda arg : {'address':int(arg[0], 16), 'size':int(arg[1], 16)}, re.findall(r'\.text\.\S+\n?\s+(0x\w+)[ \t]+(0x\w+)[ \t]+.+[/\\]'+module+r'(\(.+\.o\))?\r?\n', mem_map))
            sec_rodata  = map(lambda arg : {'address':int(arg[0], 16), 'size':int(arg[1], 16)}, re.findall(r'\.rodata\.\S+\n?\s+(0x\w+)[ \t]+(0x\w+)[ \t]+.+[/\\]'+module+r'(\(.+\.o\))?\r?\n', mem_map))
            sec_data    = map(lambda arg : {'address':int(arg[0], 16), 'size':int(arg[1], 16)}, re.findall(r'\.data\.\S+\n?\s+(0x\w+)[ \t]+(0x\w+)[ \t]+.+[/\\]'+module+r'(\(.+\.o\))?\r?\n', mem_map))
            sec_bss     = map(lambda arg : {'address':int(arg[0], 16), 'size':int(arg[1], 16)}, re.findall(r'\.bss\.\S+\n?\s+(0x\w+)[ \t]+(0x\w+)[ \t]+.+[/\\]'+module+r'(\(.+\.o\))?\r?\n', mem_map))
            
            sec_ro      = list(sec_text) + list(sec_rodata);
            sec_rw      = list(sec_data) + list(sec_bss);

            ro_size = 0
            rw_size = 0
            for sec in sec_ro:
                ro_size += sec['size']
            for sec in sec_rw:
                rw_size += sec['size']

            total_ro += ro_size
            total_rw += rw_size

            map_lines.append(r'| %-40s | %-8d  | %-8d |'%(re.sub('\.[ao]','',module)[:40],ro_size,rw_size))

    print('\n                        AOS MEMORY MAP                            ')
    print('|=================================================================|')
    print('| %-40s | %-8s  | %-8s |'%('MODULE','RO Size','RW Size'))
    print('|=================================================================|')
    for line in map_lines:
        print(line)
    print('|=================================================================|')
    print('| %-40s | %-8d  | %-8d |'%('TOTAL (bytes)', total_ro, total_rw))
    print('|=================================================================|')

def change_dir_to_solution(yoc_path):
    def getSolutionAndBoard(yoc_path):
        config_file = os.path.join(yoc_path, ".config")
        if os.path.isfile(config_file) and os.path.isfile(os.path.join(yoc_path, ".aos")):
            conf = yaml_load(config_file)
            return conf["solution"], conf["board"]
        return None, None
    cur_dir = os.getcwd()
    # enter into solution directory if in the workspace root directory
    if cur_dir == yoc_path:
        solution, board = getSolutionAndBoard(yoc_path)
        if solution:
            new_dir = os.path.join(cur_dir, "solutions", solution)
            if os.path.isdir(new_dir):
                os.chdir(new_dir)
                put_string("change dir to %s." % new_dir)
    return cur_dir

def check_remote_version():
    need_check = False
    new_version = False
    ver_file = home_path('.aliot/version')
    if os.path.isfile(ver_file):
        modified_time = os.stat(ver_file).st_mtime
        now = time.time()
        if modified_time > now or (now - modified_time) > 4 * 3600:
            need_check = True
    else:
        need_check = True
        aliot_dir = home_path('.aliot')
        if not os.path.exists(aliot_dir):
            os.mkdir(aliot_dir)
    if need_check:
        put_string("\n\nChecking latest version of aos-tools from pip server...")
        script_process = subprocess.Popen("python -m pip list --outdated", shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
        lines = script_process.stdout.readlines()
        pattern = re.compile("aos-tools\s+([0-9\.]*)\s+([0-9\.]*)")
        for text in lines:
            try:
                encode = chardet.detect(text)
                text = text.decode(encode["encoding"]).strip()
            except UnicodeDecodeError:
                text = " "
                pass
            if text.find("aos-tools") >= 0:
                match = pattern.search(text)
                if match:
                    with open(ver_file, 'w') as f:
                        f.write(text)
                    put_string("The lastest version aos-tools is %s. You can upgrade it as below:" % match.group(2))
                    put_string("python -m pip install -U aos-tools")
                    new_version = True
        if not new_version:
            with open(ver_file, 'w') as f:
                f.write("latest")
            put_string("It is already latest version.")





# -*- coding:utf-8 -*-
#
# Copyright (C) 2019-2020 Alibaba Group Holding Limited


from __future__ import print_function
import os
import re

from .log import logger
from .tools import *
from .package import *
from .component import *


class Variables:
    def __init__(self):
        self.vars = {}
        pass

    def set(self, k, v, lower=False):
        self.vars[k] = v
        if lower:
            self.vars[k.lower()] = v.lower()

    def get(self, var):
        if var in self.vars:
            return self.vars[var]

    def convert(self, var):
        if var == None or len(var) == 0:
            return None

        v = var.split('?')
        if len(v) > 1:
            x = re.search('<(.+?)>', v[1], re.M | re.I)
            if x:
                x = x.group(1).strip()
                if(x.startswith('!')):
                    x = self.get(x[1:])
                    if str(x) not in ['n', 'no', 'f', 'false', '0', False, 0, 'None']:
                        return None
                    var = v[0].strip()
                else:
                    x = self.get(x)
                    if str(x) not in ['y', 'yes', 't', 'true', '1', True, 1]:
                        return None
                    var = v[0].strip()

        x = re.findall('<(.+?)>', var)
        for key in x:
            value = self.get(key)
            if value != None:
                var = var.replace('<'+key+'>', self.vars[key])
            else:
                put_string('Not found variable:', var)
        return var

    def items(self):
        return self.vars.items()


class Solution:
    def __init__(self, components, compiler='gcc'):
        self.variables = Variables()
        self.solution_component = None       # 当前 solution 组件
        self.board_component = None          # 当前开发板组件
        self.chip_component = None           # 当前芯片组件
        self.components = components         # 使用到的组件
        self.cond_components = []            # 依赖某些条件的组件列表
        self.global_includes = []
        self.libs = []
        self.libpath = []
        self.depend_libs = []
        self.org_defines = {}
        self.defines = {}
        self.ASFLAGS = []
        self.CFLAGS = []
        self.CXXFLAGS = []
        self.LINKFLAGS = []
        self.ICC_ASFLAGS = []
        self.ICC_CFLAGS = []
        self.ICC_CXXFLAGS = []
        self.ICC_LINKFLAGS = []
        self.ARMCC_ASFLAGS = []
        self.ARMCC_CFLAGS = []
        self.ARMCC_CXXFLAGS = []
        self.ARMCC_LINKFLAGS = []
        self.ld_script = ''
        self.ld_script_component = None
        self.cpu_name = ''
        self.toolchain_prefix = ''
        self.flash_program = ''
        self.program_data_files = []
        self.debug_server = ''

        for comp in self.components:
            if len(comp.conditions) > 0:
                self.cond_components.append(comp)
                # print("-------%s,%s" % (comp.name, str(comp.conditions)))
        components = ComponentGroup()
        for a in self.components:
            if a not in self.cond_components:
                components.add(a)
        self.components = components

        set_compiler(compiler)
        self.variables.set('compiler', compiler)
        # find current solution & board component
        for component in self.components:
            if component.path == os.getcwd() and component.type == 'solution':
                self.solution_component = component
                if self.solution_component.hw_info.board_name:
                    self.board_component = self.components.get(
                        self.solution_component.hw_info.board_name)
                    if not self.board_component:
                        put_string("Not found in `%s` components that the current project depends on. " %
                                   self.solution_component.hw_info.board_name)
                        exit(-1)

                    self.chip_component = self.components.get(
                        self.board_component.hw_info.chip_name)
                    if not self.chip_component:
                        put_string("Not found in `%s` components that the current project depends on. " %
                                   self.board_component.hw_info.chip_name)
                        exit(-1)
                elif self.solution_component.hw_info.chip_name:
                    self.chip_component = self.components.get(self.solution_component.hw_info.chip_name)
                    if not self.chip_component:
                        put_string("Not found in `%s` components that the current project depends on. " %
                                   self.board_component.hw_info.chip_name)
                        exit(-1)
                break

        if not self.solution_component:
            put_string('The current directory is not a solution directory.')
            exit(-1)

        if (not self.board_component) and (not self.solution_component.hw_info.cpu_name) and (not self.solution_component.hw_info.ld_script):
            for name in self.solution_component.depends:
                if type(name) == dict:
                    name = list(name.keys())[0]
                component = self.components.get(name)
                if component:
                    if component.type == 'board':
                        self.board_component = component
                        self.solution_component.hw_info.board_name = self.board_component.name
                        self.chip_component = self.components.get(
                            self.board_component.hw_info.chip_name)
                        if self.chip_component:
                            self.solution_component.hw_info.chip_name = self.chip_component.name
                        else:
                            put_string("Not found chip component: " +
                                       self.board_component.hw_info.chip_name)
                            exit(-1)
                        break
                else:
                    put_string("`%s` not found, please install it: aos install %s" %
                               (name, name))
                    exit(-1)
            if not self.board_component:
                logger.warning(
                    'Not found board component in depends')

        if self.board_component:
            self.board_component.need_build = True
        if self.chip_component:
            self.chip_component.need_build = True

        components = ComponentGroup()
        for component in self.components:
            if component.need_build:
                components.add(component)
        self.components = components

        def cpu_set_variable(cpu, arch):
            def gen_cpu_defs(cpu):
                c = cpu.upper()
                c = c.replace('-', '').replace('.', '')
                c = '__CPU_' + c + '__'
                return c
            def gen_arch_defs(arch):
                a = arch.upper()
                a = '__ARCH_' + a + '__'
                return a

            self.cpu_name = cpu
            self.variables.set('CPU', cpu.upper(), True)
            if arch == '' or arch.upper() == 'CSKY':
                self.variables.set('cpu_num', cpu_id(cpu))
                self.variables.set('ARCH', 'CSKY', True)
            elif arch.upper() == 'ARM':
                self.variables.set('cpu_num', arm_cpu_id(cpu))
                self.variables.set('ARCH', arch, True)
            elif arch.upper() == 'RISCV':
                self.variables.set('cpu_num', riscv_cpu_id(cpu))
                self.variables.set('ARCH', arch, True)
            else:
                logger.error("not support arch:%s yet" % arch)
                exit(-1)
            # add cpu and arch defines,like __CPU_COTEXM0__ and __ARCH_ARM__
            self.variables.set(gen_arch_defs(arch), 1)
            self.variables.set(gen_cpu_defs(cpu), 1)

        def add_defines(component):
            for k, v in component.defconfig.items():
                self.variables.set(k, v)
                self.defines[k] = v
                self.org_defines[k] = v

        def update_variables():
            # copy defines
            for component in self.components:
                if component.type == 'common':
                    add_defines(component)
            for component in self.components:
                if component.type == 'kernel':
                    add_defines(component)
            for component in self.components:
                if component.type == 'drv_core' or component.type == 'drv_peripheral' or component.type == 'drv_external_device':
                    add_defines(component)
            for component in self.components:
                if component.type == 'arch':
                    add_defines(component)
            for component in self.components:
                if component.type == 'chip':
                    add_defines(component)
            for component in self.components:
                if component.type == 'board':
                    add_defines(component)
            for component in self.components:
                if component.type == 'solution':
                    add_defines(component)

            solution_hw_info = HardwareInfo({})
            for component in [self.chip_component, self.board_component, self.solution_component]:
                if component:
                    solution_hw_info.merge(component.hw_info)

                    for c in component.build_config.cflag.split():
                        self.CFLAGS.append(c)
                    for c in component.build_config.asmflag.split():
                        self.ASFLAGS.append(c)
                    for c in component.build_config.cxxflag.split():
                        self.CXXFLAGS.append(c)
                    for c in component.build_config.ldflag.split():
                        self.LINKFLAGS.append(c)
                    
                    for c in component.build_config.icc_cflag.split():
                        self.ICC_CFLAGS.append(c)
                    for c in component.build_config.icc_asmflag.split():
                        self.ICC_ASFLAGS.append(c)
                    for c in component.build_config.icc_cxxflag.split():
                        self.ICC_CXXFLAGS.append(c)
                    for c in component.build_config.icc_ldflag.split():
                        self.ICC_LINKFLAGS.append(c)
                    
                    for c in component.build_config.armcc_cflag.split():
                        self.ARMCC_CFLAGS.append(c)
                    for c in component.build_config.armcc_asmflag.split():
                        self.ARMCC_ASFLAGS.append(c)
                    for c in component.build_config.armcc_cxxflag.split():
                        self.ARMCC_CXXFLAGS.append(c)
                    for c in component.build_config.armcc_ldflag.split():
                        self.ARMCC_LINKFLAGS.append(c)

            # 根据 cpu_id 加载配置
            if solution_hw_info.cpu_id in solution_hw_info.cpu_list:
                for k, v in solution_hw_info.cpu_list[solution_hw_info.cpu_id].items():
                    solution_hw_info.__dict__[k] = v

            # board 的hw_info 次优先
            if self.board_component and self.board_component.hw_info:
                solution_hw_info.merge(self.board_component.hw_info, True)
            # soution 的hw_info 优先
            solution_hw_info.merge(self.solution_component.hw_info, True)

            if solution_hw_info.cpu_name == '':
                logger.error("not found cpu_id, please set cpu_id")
                exit(-1)

            if solution_hw_info.cpu_name:
                cpu_set_variable(solution_hw_info.cpu_name,
                                solution_hw_info.arch_name)

            if solution_hw_info.cpu_id:
                self.variables.set(solution_hw_info.cpu_id.upper(), True)

            if solution_hw_info.vendor_name:
                self.variables.set(
                    'VENDOR', solution_hw_info.vendor_name.upper(), True)

            if solution_hw_info.chip_name:
                self.variables.set(
                    'CHIP', solution_hw_info.chip_name.upper(), True)

            if solution_hw_info.board_name:
                self.variables.set(
                    'BOARD', solution_hw_info.board_name.upper(), True)

            if self.chip_component:
                self.variables.set('CHIP_PATH', self.chip_component.path)

            if self.board_component:
                self.variables.set('BOARD_PATH', self.board_component.path)

            self.variables.set('SOLUTION_PATH', self.solution_component.path)
            self.yoc_path = os.path.join(self.solution_component.path, 'aos_sdk')
            self.lib_path = os.path.join(self.yoc_path, 'lib')
            self.libpath.append(self.lib_path)
            self.flash_program = solution_hw_info.flash_program
            self.program_data_files = solution_hw_info.program_data_files
            self.debug_server = solution_hw_info.debug_server

            self.global_includes.append(os.path.join(
                self.solution_component.path, 'include'))

            for component in self.components:
                if len(component.source_files) > 0 and component.name not in self.libs:
                    self.libs.append(component.name)
                    # if component.name is start with lib, scons will not add prefix: lib
                    if component.name.startswith('lib'):
                        self.depend_libs.append(os.path.join(
                            self.lib_path, component.name + '.a'))
                    else:
                        self.depend_libs.append(os.path.join(
                            self.lib_path, 'lib' + component.name + '.a'))

                component.variable_convert(self.variables)

                # include
                for path in component.build_config.include_updated:
                    if path not in self.global_includes:
                        self.global_includes.append(path)

                # libpath
                for path in component.build_config.libpath_updated:
                    if path not in self.libpath:
                        self.libpath.append(path)

                # libs
                for lib in component.build_config.libs_updated:
                    if lib not in self.libs:
                        self.libs.append(lib)

            self.toolchain_prefix = solution_hw_info.toolchain_prefix
            self.ld_script = solution_hw_info.ld_script
            self.ld_script = self.variables.convert(self.ld_script)

            if self.solution_component.hw_info.ld_script:
                self.ld_script_component = self.solution_component
            elif self.board_component and self.board_component.hw_info.ld_script:
                self.ld_script_component = self.board_component
            elif self.chip_component and self.chip_component.hw_info.ld_script:
                self.ld_script_component = self.chip_component
            elif self.chip_component and self.ld_script:
                # multi cpu
                self.ld_script_component = self.chip_component
            else:
                logger.error("not found LD script file, please set ld_script")
                exit(-1)

            # append variables to defines
            for k, v in self.variables.items():
                if k.isupper():
                    if k.startswith('__CPU_') or k.startswith('__ARCH_'):
                        self.defines[k] = v
                    elif k in ['CPU', 'ARCH', 'VENDOR', 'CHIP', 'BOARD', 'CHIP_PATH', 'BOARD_PATH', 'SOLUTION_PATH']:
                        self.defines['__AOS_' + k + '__'] = v

        update_variables()

        def check_variable(values=[]):
            # check and condition: [v1, v2, v3...], AND operation
            for v in values:
                if(v.startswith('!')):
                    a = self.variables.get(v[1:])
                    if str(a) not in ['n', 'no', 'f', 'false', '0', False, 0, 'None']:
                        return False
                else:
                    a = self.variables.get(v)
                    if str(a) not in ['y', 'yes', 't', 'true', '1', True, 1]:
                        return False

            return True

        def check_variables(values=[]):
            # check and condition: [['aa'], ['bb', 'cc'], ['dd', 'ee']] or [['ff']], OR operation
            for v in values:
                if check_variable(v):
                    return True

            return False

        for cc in self.cond_components:
            if check_variables(cc.conditions):
                self.components.add(cc)
                update_variables()

    def install(self):
        for component in self.components:
            component.install(self.yoc_path)

    def show(self, show_component=True, show_libs=True, show_include=True, show_define=True, show_veriable=True, show_flags=True):
        if show_component:
            put_string("[component]")
            for c in self.components:
                c.show(4)

            put_string()
            if self.solution_component:
                put_string("    solution component: " +
                           self.solution_component.name)
            if self.board_component:
                put_string("    board component   : " +
                           self.board_component.name)
            if self.chip_component:
                put_string("    chip component    : " +
                           self.chip_component.name)

        if show_libs:
            put_string("[libs]")
            for v in self.libs:
                put_string('    ' + v)

            put_string("[libpath]")
            for v in self.libpath:
                put_string('    ' + v)

        if show_include:
            put_string("[global_includes]")
            for v in self.global_includes:
                put_string('    ' + v)

        if show_define:
            put_string("[defines]")
            for k, v in self.defines.items():
                put_string('    ', k, '=', v)

        if show_veriable:
            put_string("[variables]")
            for k, v in self.variables.items():
                put_string('    %s = %s' % (k, v))

        if show_flags:
            put_string("\nGCC COMPILER")
            put_string("ASFLAGS   :" + ' '.join(self.ASFLAGS))
            put_string("CFLAGS   :" + ' '.join(self.CFLAGS))
            put_string("CXXFLAGS  :" + ' '.join(self.CXXFLAGS))
            put_string("LINKFLAGS :" + ' '.join(self.LINKFLAGS))
            put_string('ld_script : ' + self.ld_script)
            put_string('toolchain : ' + self.toolchain_prefix)
            # put_string("\nIAR COMPILER")
            # put_string("ASFLAGS   :" + ' '.join(self.ICC_ASFLAGS))
            # put_string("CFLAGS   :" + ' '.join(self.ICC_CFLAGS))
            # put_string("CXXFLAGS  :" + ' '.join(self.ICC_CXXFLAGS))
            # put_string("LINKFLAGS :" + ' '.join(self.ICC_LINKFLAGS))
            # put_string("\nKEIL COMPILER")
            # put_string("ASFLAGS   :" + ' '.join(self.ARMCC_ASFLAGS))
            # put_string("CFLAGS   :" + ' '.join(self.ARMCC_CFLAGS))
            # put_string("CXXFLAGS  :" + ' '.join(self.ARMCC_CXXFLAGS))
            # put_string("LINKFLAGS :" + ' '.join(self.ARMCC_LINKFLAGS))



def cpu_id(s):
    s += 'z'
    ids = ''
    for c in s:
        if ord(c) >= ord('0') and ord(c) <= ord('9'):
            ids += c
        elif ids:
            return ids


def arm_cpu_id(s):
    if s.startswith('arm'):
        return s[3:]
    else:
        arr = s.split('-')
        if len(arr) > 1:
            return arr[1]

def riscv_cpu_id(cpu):
    a = cpu.lower()
    if a == 'rv32emc':
        return 'rv32'
    elif a == 'e902' or a == 'e902m' or a == 'e902t' or a == 'e902mt':
        return 'rv32_16gpr'
    elif a == 'e906':
        return 'rv32_32gpr'
    elif a == 'e906f':
        return 'rv32f_32gpr'
    elif a == 'e906fd':
        return 'rv32fd_32gpr'
    elif a == 'c906':
        return 'rv64_32gpr'
    elif a == 'c906fd' or a == 'c906v' or a == 'c910' or a == 'c910v':
        return 'rv64fd_32gpr'
    else:
        put_string("%s not support yet." % a)
        exit(0)



# -*- coding:utf-8 -*-
#
# Copyright (C) 2019-2020 Alibaba Group Holding Limited


from __future__ import print_function

from aostools import *
from aostools.tools import get_elf_bin_file
import os
import codecs
import shutil
from serial.tools import miniterm

class Monitor(Command):
    common = True
    helpSummary = "Open a simple serial monitor."
    helpUsage = """
%prog [PORT] [BAUDRATE]
"""
    helpDescription = """
Open a simple serial monitor.
"""
    # def _Options(self, p):
    #     p.add_option('-b', '--board',
    #                  dest='board_name', action='store', type='str', default=None,
    #                  help='get burn tools from selected board')

    def Execute(self, opt, args):
        yoc = YoC()
        change_dir_to_solution(yoc.conf.yoc_path)
        config = self.get_config()
        needsave = False
        if len(args) == 2:
            config['serialport'] = args[0]
            config['baudrate'] = args[1]
            needsave = True
        if (not config['serialport']) or (not config['baudrate']):
            config["serialport"] = miniterm.ask_for_port()
            if not config['serialport']:
                print("no specified serial port")
                return 1
            if sys.version_info[0] < 3:
                config['baudrate'] = raw_input('--- Enter serial baudrate: ')
            else:
                config['baudrate'] = input('--- Enter serial baudrate: ')
            if not config['baudrate']:
                print("no specified baudrate")
                return 1
            needsave = True

        ret = os.system("pyserial-miniterm %s %s" % (config["serialport"], config["baudrate"])) >> 8
        if needsave and ret == 0:
            self.save_config(config)
        return ret

    def read_json(self, json_file):
        data = {}
        if os.path.isfile(json_file):
            with codecs.open(json_file, 'r', 'UTF-8') as f:
                data = json.load(f)
                if not data:
                    data = {}
        return data

    def write_json(self, json_file, data):
        with codecs.open(json_file, 'w', 'UTF-8') as f:
            f.write(json.dumps(data, indent=4, separators=(',', ': ')))

    def get_config(self):
        """ get configuration from .config_monitor file """ 
        config = {}
        config_file = os.path.join(os.getcwd(), '.config_monitor')
        if os.path.isfile(config_file):
            config = self.read_json(config_file)

        if 'serialport' not in config:
            config['serialport'] = ""
        if 'baudrate' not in config:
            config['baudrate'] = ""

        return config

    def save_config(self, config):
        """ save configuration to .config_monitor file""" 
        if config:
            config_file = os.path.join(os.getcwd(), '.config_monitor')
            self.write_json(config_file, config)

